@@ -171,6 +171,7 @@ save_config(void)
   gconf_client_set_int(client, GCONF_DIR "global/scrollbars_fullscreen", pref.scrollbars_fullscreen, NULL);
   gconf_client_set_int(client, GCONF_DIR "global/rotate_ccw", pref.rotate_ccw, NULL);
   gconf_client_set_int(client, GCONF_DIR "global/sort_order", pref.sort_order, NULL);
+  gconf_client_set_int(client, GCONF_DIR "global/cache_next", pref.cache_next, NULL);
 
   save_accels(client, KEY_FMT, MODS_FMT, key_asg0);
 #if USE_HILDON
@@ -194,9 +195,12 @@ gchar* gconf_client_get_string_with_default(GConfClient* client, const gchar* ke
 
 int gconf_client_get_int_with_default(GConfClient* client, const gchar* key, int def)
 {
+  GConfValue* value;
   int retval;
-  retval = gconf_client_get_int(client, key, NULL);
-  if(!retval)
+  value = gconf_client_get(client, key, NULL);
+  if(value)
+    retval = gconf_value_get_int(value);
+  else
     retval = def;
   return retval;
 }
@@ -249,6 +253,7 @@ load_config(void)
   pref.scrollbars_fullscreen = gconf_client_get_int_with_default(client, GCONF_DIR "global/scrollbars_fullscreen", 1);
   pref.rotate_ccw = gconf_client_get_int_with_default(client, GCONF_DIR "global/rotate_ccw", 0);
   pref.sort_order = gconf_client_get_int_with_default(client, GCONF_DIR "global/sort_order", 0);
+  pref.cache_next = gconf_client_get_int_with_default(client, GCONF_DIR "global/cache_next", 0);
 
   load_accels(client, KEY_FMT, MODS_FMT, key_asg0);
 #if USE_HILDON
@@ -312,6 +317,8 @@ show_config(GtkWidget *ref)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), pref.rotate_ccw ? TRUE : FALSE);
   w = lookup_widget(ref, "sort_order");
   gtk_combo_box_set_active(GTK_COMBO_BOX(w), pref.sort_order);
+  w = lookup_widget(ref, "cache_next");
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), pref.cache_next ? TRUE : FALSE);
 
 #if !USE_HILDON
   switch (pref.nav_pos) {
@@ -428,6 +435,9 @@ parse_config(GtkWidget *ref)
   if(pref.sort_order == -1)
     pref.sort_order = 0;
 
+  w = lookup_widget(ref, "cache_next");
+  pref.cache_next = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) ? 1 : 0;
+
 #if !USE_HILDON
   w = lookup_widget(ref, "conf_nav_nw_rbtn");
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
