@@ -43,6 +43,7 @@
 #include "garchivereader.h"
 #include "garchivereadercommandline.h"
 #include "garchivereaderunzip.h"
+#include "imagecache.h"
 
 #ifdef USE_HILDON
 #    include <hildon/hildon-program.h>
@@ -58,7 +59,6 @@ enum FitSettings {
 
 #define ZONE_SIZE 20
 
-GArray          *pagelist = NULL;
 int		 page_nr = 0,
   timer_id = 0;
 #if USE_HILDON
@@ -155,16 +155,16 @@ initial_setup(void)
   book_canvas = GTK_IMAGE_VIEW(lookup_widget(MainWindow, "main_canvas"));
   image_buffer = g_malloc(pref.buffer_size);
 
-  readers[0] = g_archive_reader_unzip_new(image_buffer, 
+  /*readers[0] = g_archive_reader_unzip_new(image_buffer, 
 					  pref.buffer_size, 
-					  zip_magic);
+					  zip_magic);*/
   /* Note: if you have problems with the in-memory unzipper, use this
      one instead */
-  /*  readers[0] = g_archive_reader_command_line_new(image_buffer, 
+  readers[0] = g_archive_reader_command_line_new(image_buffer, 
       pref.buffer_size,
       zip_magic,
       zip_list_command,
-      zip_decompress_command);*/
+      zip_decompress_command);
   readers[1] = g_archive_reader_command_line_new(image_buffer,
 						 pref.buffer_size,
 						 rar_magic,
@@ -206,6 +206,7 @@ initial_setup(void)
 		      goto_page);
 }
 
+
 void
 clean_up(void)
 {
@@ -214,6 +215,7 @@ clean_up(void)
     g_hash_table_unref(accel_table);
     accel_table = NULL;
   }
+  image_cache_free();
   reader = NULL;
   for(i = 0; i < NUM_READERS; ++i) {
     if(readers[i])
@@ -228,13 +230,9 @@ clean_up(void)
 void
 set_sort_strategy(void)
 {
-  int i;
-  for(i = 0; i < NUM_READERS; ++i)
-    g_archive_reader_set_sort_strategy(readers[i],
-				       (SortStrategy)pref.sort_order);
-  /* re-request the current page */
-  if(pagelist && pagelist->len > 0)
-    show_page(page_nr);
+  /* re-sort and re-request the current page */ 
+  image_cache_resort();
+  show_page(page_nr);
 }
 
 void
@@ -267,8 +265,8 @@ start_show(void)
   }
 
   reader = tmpreader;
-  pagelist = g_archive_reader_get_file_list(reader);
-  if(!pagelist) {
+  image_cache_load(reader);
+  if(image_cache_length() == 0) {
     gchar* last_error = g_archive_reader_get_last_error(reader);
     if(last_error) {
       printf("%s\n", last_error);
@@ -276,6 +274,7 @@ start_show(void)
       g_free(last_error);
     }
   }
+  
   show_page(pref.pagenumber);
   page_nr = pref.pagenumber;		/* Thanks James Reeves jreeves[at]monkeyengines.co.uk */
 }
@@ -312,7 +311,7 @@ draw_indicators(GtkWidget* canvas, int which)
   case 0:
     if(page_nr != 0)
       draw_indicators(canvas, -1);
-    if(pagelist && page_nr + pref.nrpages < pagelist->len)
+    if(page_nr + pref.nrpages < image_cache_length())
       draw_indicators(canvas, 1);
     /* set up a timer to remove the arrows after a time */
     if(timeout_id != 0) {
@@ -500,97 +499,38 @@ scroll_to(double where)
 }
 
 
-void
-request_page(int nr, GdkPixbuf **pxm, double *w, double *h)
-{
-  if(reader == NULL) {
-    return;
-  }
-  *pxm = g_archive_reader_get_page(reader, nr, w, h);
-}
-
-
 void
 show_page(int nr)
 {
-  GdkPixbuf *pxm, *pxm1 = NULL, *final = NULL;
-  char *bff;
-  double w, h, w1 = 0, h1 = 0;
+  GdkPixbuf *final = NULL;
+  gchar *bff;
+  const gchar* page_title;
   int nr_pages;
 
-  nr_pages = pagelist ? pagelist->len : 0;
+  nr_pages = image_cache_length();
 
-  if(reader == NULL || !nr_pages) {
-    printf(_("No pages loaded\n"));
+  if(!nr_pages) {
+    info_dialog(_("No pages loaded\n"));
     return;
   }
 
+  final = image_cache_request_page(nr, 
+				   pref.nrpages == 2, 
+				   pref.manga_mode,
+				   &total_w,
+				   &total_h,
+				   &page_title);
+
 #if USE_HILDON
   bff = g_strdup_printf(_("%s"),
-			(char *)g_array_index(pagelist, gchar*, nr));
+			page_title);
 #else
   bff = g_strdup_printf(_("cbrPager - %s"),
-			(char *)g_array_index(pagelist, gchar*, nr));
+			page_title);
 #endif
   gtk_window_set_title(GTK_WINDOW(MainWindow), bff);
   g_free(bff);
 
-  request_page(nr, &pxm, &w, &h);
-  if(pxm == NULL) {
-    gchar* last_error = g_archive_reader_get_last_error(reader);
-    ok_dialog(_("File error"), last_error);
-    g_free(last_error);
-    return;
-  }
-  if (pref.nrpages == 2 && nr+1 < nr_pages) {
-    int bps, bps1;
-    request_page(nr+1, &pxm1, &w1, &h1);
-    if(pxm1 == NULL) {
-      gchar* last_error = g_archive_reader_get_last_error(reader);
-      ok_dialog(_("File error"), last_error);
-      g_free(last_error);
-      g_object_unref(pxm);
-      return;
-    }
-    bps = gdk_pixbuf_get_bits_per_sample(pxm);
-    bps1 = gdk_pixbuf_get_bits_per_sample(pxm1);
-    
-    final = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
-			   FALSE,
-			   MAX(bps, bps1),
-			   w + w1,
-			   MAX(h, h1));
-    gdk_pixbuf_fill(final, 0xffffffff);
-    if(pref.manga_mode) {
-      gdk_pixbuf_copy_area(pxm1,
-			   0, 0, w1, h1,
-			   final,
-			   0, 0);
-      g_object_unref(pxm1);
-      gdk_pixbuf_copy_area(pxm,
-			   0, 0, w, h,
-			   final,
-			   w1, 0);
-      g_object_unref(pxm);
-    } else {
-      gdk_pixbuf_copy_area(pxm,
-			   0, 0, w, h,
-			   final,
-			   0, 0);
-      g_object_unref(pxm);
-      gdk_pixbuf_copy_area(pxm1,
-			   0, 0, w1, h1,
-			   final,
-			   w, 0);
-      g_object_unref(pxm1);
-    }
-  } else {
-    final = pxm;
-  }
-  
-  
-  total_w = w + w1;
-  total_h = h > h1 ? h : h1;
   if(pref.auto_rotate && 
      ((total_w >= total_h && rotated) ||
       (total_w < total_h && !rotated)))
@@ -599,10 +539,6 @@ show_page(int nr)
   gtk_image_view_set_pixbuf(book_canvas, final, zoom_fit == FitSettings_PAGE ? TRUE : FALSE);
   reset_fit_settings();
 
-  /* now the image view is holding the pixmap, no need to keep it around */
-  g_object_unref(final);
-
-
   /* save last requested page */
   pref.pagenumber = nr;
   save_config();
@@ -638,7 +574,7 @@ fit_width(int w)
 void
 next_page(int adv, gboolean show_indicator)
 {
-  int by = adv, nr_pages = pagelist ? pagelist->len : 0;
+  int by = adv, nr_pages = image_cache_length();
 
   if (by == 0) by = pref.nrpages;
   if (page_nr == (nr_pages - by)) {
@@ -695,10 +631,9 @@ goto_begin(void)
 void
 goto_end(void)
 {
-  if(!pagelist)
-    return;
-  page_nr = pagelist->len - pref.nrpages;
-  show_page(page_nr);
+  page_nr = image_cache_length() - pref.nrpages;
+  if(page_nr > 0)
+    show_page(page_nr);
 }
 
 
@@ -1016,7 +951,7 @@ main_canvas_buttonpressed(GdkEventButton *event)
 
   if (event->type != GDK_BUTTON_PRESS) return;
 
-  nr_pages = pagelist ? pagelist->len : 0;
+  nr_pages = image_cache_length();
 
   switch (event->button) {
   case 1:				// Left button
@@ -1154,7 +1089,7 @@ about_box(GtkWidget* main_window)
 void
 goto_page()
 {
-  int nr_pages = pagelist ? pagelist->len : 0;
+  int nr_pages = image_cache_length();
   int result = invoke_goto_page_dialog(MainWindow, 
 				       page_nr, 
 				       nr_pages - pref.nrpages);
